//
//  GameScene.swift
//  DouShouQi_App
//
//  Created by Nathan Verdier on 28/05/2024.
//

import Foundation
import SpriteKit
import DouShouQiModel

class GameScene : SKScene {
    var game: Game = try! Game(withRules: ClassicRules(), andPlayer1: RandomPlayer(withName: "Rayhan", andId: .player1)!, andPlayer2: RandomPlayer(withName: "Rémi", andId: .player2)!)
    
    let pieces: [Owner : [Animal:SpriteMeeple]] = [
        .player1: [
            .rat: SpriteMeeple(imageNamed: AppImages.Rat, size: CGSize(width: 100, height: 100), backgroundColor: Colors.MeepleP1!),
            .cat: SpriteMeeple(imageNamed: AppImages.Cat, size: CGSize(width: 100, height: 100), backgroundColor: Colors.MeepleP1!),
            .dog: SpriteMeeple(imageNamed: AppImages.Dog, size: CGSize(width: 100, height: 100), backgroundColor: Colors.MeepleP1!),
            .wolf: SpriteMeeple(imageNamed: AppImages.Wolf, size: CGSize(width: 100, height: 100), backgroundColor: Colors.MeepleP1!),
            .leopard: SpriteMeeple(imageNamed: AppImages.Leopard, size: CGSize(width: 100, height: 100), backgroundColor: Colors.MeepleP1!),
            .lion: SpriteMeeple(imageNamed: AppImages.Lion, size: CGSize(width: 100, height: 100), backgroundColor: Colors.MeepleP1!),
            .tiger: SpriteMeeple(imageNamed: AppImages.Tigger, size: CGSize(width: 100, height: 100), backgroundColor: Colors.MeepleP1!),
            .elephant: SpriteMeeple(imageNamed: AppImages.Elephant, size: CGSize(width: 100, height: 100), backgroundColor: Colors.MeepleP1!),
        ],
        .player2: [
            .rat: SpriteMeeple(imageNamed: AppImages.Rat, size: CGSize(width: 100, height: 100), backgroundColor: .blue, imageRotation: 180),
            .cat: SpriteMeeple(imageNamed: AppImages.Cat, size: CGSize(width: 100, height: 100), backgroundColor: .blue, imageRotation: 180),
            .dog: SpriteMeeple(imageNamed: AppImages.Dog, size: CGSize(width: 100, height: 100), backgroundColor: .blue, imageRotation: 180),
            .wolf: SpriteMeeple(imageNamed: AppImages.Wolf, size: CGSize(width: 100, height: 100), backgroundColor: .blue, imageRotation: 180),
            .leopard: SpriteMeeple(imageNamed: AppImages.Leopard, size: CGSize(width: 100, height: 100), backgroundColor: .blue, imageRotation: 180),
            .lion: SpriteMeeple(imageNamed: AppImages.Lion, size: CGSize(width: 100, height: 100), backgroundColor: .blue, imageRotation: 180),
            .tiger: SpriteMeeple(imageNamed: AppImages.Tigger, size: CGSize(width: 100, height: 100), backgroundColor: .blue, imageRotation: 180),
            .elephant: SpriteMeeple(imageNamed: AppImages.Elephant, size: CGSize(width: 100, height: 100), backgroundColor: .blue, imageRotation: 180),
        ],
    ]
    
    let imageBoard: SKSpriteNode = SKSpriteNode(imageNamed: AppImages.boardGame)
    
    override init(size: CGSize) {
        super.init(size: size)
        imageBoard.size = size
        //self.scaleMode = .aspectFit
        self.anchorPoint = CGPoint(x: 0.5, y: 0.5)
        self.backgroundColor = .white
        
        self.addChild(imageBoard)
        
        for piece in pieces.flatMap({owner, pieces in pieces.values}) {
            self.addChild(piece)
        }
        
        initializeBoard(game.board)
        //displayBoard(game.board)
    }
    
    func initializeBoard(_ board: Board) {
        
        /*for (lineIndex, currentLine) in game.board.grid.enumerated() {
            for (cellIndex, currentCell) in currentLine.enumerated() {
                if (currentCell.piece != nil){
                    pieces[currentCell.piece!.owner]?[currentCell.piece!.animal]?.cellPosition = CGPoint(x: cellIndex, y: lineIndex)
                }
            }
        }*/
        
        for (lineIndex, currentLine) in game.board.grid.enumerated() {
            for (cellIndex, currentCell) in currentLine.enumerated() {
                if let piece = currentCell.piece {
                    pieces[piece.owner]?[piece.animal]?.cellPosition = CGPoint(x: cellIndex-3, y: lineIndex-4)
                    pieces[piece.owner]?[piece.animal]?.cellPosition = CGPoint(x: 0, y: 0)
                    print("line :", lineIndex, " cologne:  ", cellIndex)
                }
            }
        }

        
        /*var new_array = game.board.grid.flatMap(pieces => {
            pieces.forEach( => {
                if (joueur1.pieces.includes(piece)) {
                    console.log(`La pièce ${piece} appartient à joueur1`);
                } else if (joueur2.pieces.includes(piece)) {
                    console.log(`La pièce ${piece} appartient à joueur2`);
                }
            });
            // Retourner les pièces pour le new_array (si nécessaire)
            return pieces;
        });*/

/*
        let player1AnimalsLigne1: [Animal] = [.elephant, .wolf, .tiger, .rat]
        let player1AnimalsLigne2: [Animal] = [.cat, .dog]
        let player1AnimalsLigne3: [Animal] = [.leopard, .lion]
        
        let player2AnimalsLigne1: [Animal] = [.elephant, .wolf, .tiger, .rat]
        let player2AnimalsLigne2: [Animal] = [.cat, .dog]
        let player2AnimalsLigne3: [Animal] = [.leopard, .lion]
        
        for (index, animal) in player1AnimalsLigne1.enumerated() {
            pieces[.player1]?[animal]?.cellPosition = CGPoint(x: index-3, y: -2)
            pieces[.player1]?[animal]?.cellPosition = CGPoint(x: index, y: 1)
        }
        
        for (index, animal) in player1AnimalsLigne2.enumerated() {
            pieces[.player1]?[animal]?.cellPosition = CGPoint(x: index-3, y: -3)
            pieces[.player1]?[animal]?.cellPosition = CGPoint(x: index, y: 1)
        }
        
        for (index, animal) in player1AnimalsLigne3.enumerated() {
            pieces[.player1]?[animal]?.cellPosition = CGPoint(x: index-3, y: -3)
            pieces[.player1]?[animal]?.cellPosition = CGPoint(x: index, y: 1)
        }
        
        for (index, animal) in player2AnimalsLigne1.enumerated() {
            pieces[.player2]?[animal]?.cellPosition = CGPoint(x: index-3, y: 3)
            pieces[.player2]?[animal]?.cellPosition = CGPoint(x: index, y:  1)
        }
        
        for (index, animal) in player2AnimalsLigne2.enumerated() {
            pieces[.player2]?[animal]?.cellPosition = CGPoint(x: index-3, y: 3)
            pieces[.player2]?[animal]?.cellPosition = CGPoint(x: index, y:  1)
        }
        
        for (index, animal) in player2AnimalsLigne3.enumerated() {
            pieces[.player2]?[animal]?.cellPosition = CGPoint(x: index-3, y: 3)
            pieces[.player2]?[animal]?.cellPosition = CGPoint(x: index, y:  1)
        }*/
    }
    
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder);
    }
}
